"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const asKeyMapSync_1 = require("./asKeyMapSync");
/**
 * Sorts values in an Iterable based on key and a key comparer.
 * @param source Iterable
 * @param keySelector Key Selector
 * @param ascending Ascending or Descending Sort
 * @param comparer Key Comparer for Sorting. Optional.
 * @returns Async Iterable Iterator
 */
function asSortedKeyValuesSync(source, keySelector, ascending, comparer) {
    return tslib_1.__asyncGenerator(this, arguments, function* asSortedKeyValuesSync_1() {
        const map = yield tslib_1.__await(asKeyMapSync_1.asKeyMapSync(source, keySelector));
        const sortedKeys = [...map.keys()].sort(comparer ? comparer : undefined);
        if (ascending) {
            for (let i = 0; i < sortedKeys.length; i++) {
                yield yield tslib_1.__await(map.get(sortedKeys[i]));
            }
        }
        else {
            for (let i = sortedKeys.length - 1; i >= 0; i--) {
                yield yield tslib_1.__await(map.get(sortedKeys[i]));
            }
        }
    });
}
exports.asSortedKeyValuesSync = asSortedKeyValuesSync;
