"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const BasicAsyncEnumerable_1 = require("../BasicAsyncEnumerable");
/**
 * Projects each element of a sequence to an IAsyncEnumerable<T> and flattens the resulting sequences into one sequence.
 * @param source A sequence of values to project.
 * @param selector A transform function to apply to each element.
 * @returns An IAsyncEnumerable<T> whose elements are the result of invoking the
 * one-to-many transform function on each element of the input sequence.
 */
function selectManyAsync(source, selector) {
    if (selector.length === 1) {
        const iterator = function () {
            return tslib_1.__asyncGenerator(this, arguments, function* () {
                var e_1, _a;
                try {
                    for (var source_1 = tslib_1.__asyncValues(source), source_1_1; source_1_1 = yield tslib_1.__await(source_1.next()), !source_1_1.done;) {
                        const value = source_1_1.value;
                        const many = yield tslib_1.__await(selector(value));
                        for (const innerValue of many) {
                            yield yield tslib_1.__await(innerValue);
                        }
                    }
                }
                catch (e_1_1) { e_1 = { error: e_1_1 }; }
                finally {
                    try {
                        if (source_1_1 && !source_1_1.done && (_a = source_1.return)) yield tslib_1.__await(_a.call(source_1));
                    }
                    finally { if (e_1) throw e_1.error; }
                }
            });
        };
        return new BasicAsyncEnumerable_1.BasicAsyncEnumerable(iterator);
    }
    else {
        const iterator = function () {
            return tslib_1.__asyncGenerator(this, arguments, function* () {
                var e_2, _a;
                let index = 0;
                try {
                    for (var source_2 = tslib_1.__asyncValues(source), source_2_1; source_2_1 = yield tslib_1.__await(source_2.next()), !source_2_1.done;) {
                        const value = source_2_1.value;
                        const many = yield tslib_1.__await(selector(value, index));
                        for (const innerValue of many) {
                            yield yield tslib_1.__await(innerValue);
                        }
                        index++;
                    }
                }
                catch (e_2_1) { e_2 = { error: e_2_1 }; }
                finally {
                    try {
                        if (source_2_1 && !source_2_1.done && (_a = source_2.return)) yield tslib_1.__await(_a.call(source_2));
                    }
                    finally { if (e_2) throw e_2.error; }
                }
            });
        };
        return new BasicAsyncEnumerable_1.BasicAsyncEnumerable(iterator);
    }
}
exports.selectManyAsync = selectManyAsync;
