"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
/**
 * Compares two async iterables to see if they are equal using a async comparer function.
 * @param first First Sequence
 * @param second Second Sequence
 * @param comparer Async Comparer
 * @returns Whether or not the two iterations are equal
 */
function sequenceEqualsAsync(first, second, comparer) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const firstIterator = first[Symbol.asyncIterator]();
        const secondIterator = second[Symbol.asyncIterator]();
        let results = yield Promise.all([firstIterator.next(), secondIterator.next()]);
        let firstResult = results[0];
        let secondResult = results[1];
        while (!firstResult.done && !secondResult.done) {
            if ((yield comparer(firstResult.value, secondResult.value)) === false) {
                return false;
            }
            results = yield Promise.all([firstIterator.next(), secondIterator.next()]);
            firstResult = results[0];
            secondResult = results[1];
        }
        return firstResult.done && secondResult.done;
    });
}
exports.sequenceEqualsAsync = sequenceEqualsAsync;
