"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const BasicAsyncEnumerable_1 = require("../BasicAsyncEnumerable");
/**
 * Returns elements from a sequence as long as a specified condition is true.
 * The element's index is used in the logic of the predicate function.
 * @param source The sequence to return elements from.
 * @param predicate A function to test each source element for a condition;
 * the second parameter of the function represents the index of the source element.
 * @returns An IAsyncEnumerable<T> that contains elements from the input sequence
 * that occur before the element at which the test no longer passes.
 */
function takeWhile(source, predicate) {
    if (predicate.length === 1) {
        return takeWhile_1(source, predicate);
    }
    else {
        return takeWhile_2(source, predicate);
    }
}
exports.takeWhile = takeWhile;
function takeWhile_1(source, predicate) {
    function iterator() {
        return tslib_1.__asyncGenerator(this, arguments, function* iterator_1() {
            var e_1, _a;
            try {
                for (var source_1 = tslib_1.__asyncValues(source), source_1_1; source_1_1 = yield tslib_1.__await(source_1.next()), !source_1_1.done;) {
                    const item = source_1_1.value;
                    if (predicate(item)) {
                        yield yield tslib_1.__await(item);
                    }
                    else {
                        break;
                    }
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (source_1_1 && !source_1_1.done && (_a = source_1.return)) yield tslib_1.__await(_a.call(source_1));
                }
                finally { if (e_1) throw e_1.error; }
            }
        });
    }
    return new BasicAsyncEnumerable_1.BasicAsyncEnumerable(iterator);
}
function takeWhile_2(source, predicate) {
    function iterator() {
        return tslib_1.__asyncGenerator(this, arguments, function* iterator_2() {
            var e_2, _a;
            let index = 0;
            try {
                for (var source_2 = tslib_1.__asyncValues(source), source_2_1; source_2_1 = yield tslib_1.__await(source_2.next()), !source_2_1.done;) {
                    const item = source_2_1.value;
                    if (predicate(item, index++)) {
                        yield yield tslib_1.__await(item);
                    }
                    else {
                        break;
                    }
                }
            }
            catch (e_2_1) { e_2 = { error: e_2_1 }; }
            finally {
                try {
                    if (source_2_1 && !source_2_1.done && (_a = source_2.return)) yield tslib_1.__await(_a.call(source_2));
                }
                finally { if (e_2) throw e_2.error; }
            }
        });
    }
    return new BasicAsyncEnumerable_1.BasicAsyncEnumerable(iterator);
}
