"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const BasicAsyncEnumerable_1 = require("../BasicAsyncEnumerable");
/**
 * Produces the set union of two sequences by using a specified IAsyncEqualityComparer<T>.
 * @param first An AsyncIterable<T> whose distinct elements form the first set for the union.
 * @param second An AsyncIterable<T> whose distinct elements form the second set for the union.
 * @param comparer The IAsyncEqualityComparer<T> to compare values.
 * @returns An IAsyncEnumerable<T> that contains the elements from both input sequences, excluding duplicates.
 */
function unionAsync(first, second, comparer) {
    function iterator() {
        return tslib_1.__asyncGenerator(this, arguments, function* iterator_1() {
            var e_1, _a;
            const result = [];
            for (const source of [first, second]) {
                try {
                    for (var source_1 = tslib_1.__asyncValues(source), source_1_1; source_1_1 = yield tslib_1.__await(source_1.next()), !source_1_1.done;) {
                        const value = source_1_1.value;
                        let exists = false;
                        for (const resultValue of result) {
                            if ((yield tslib_1.__await(comparer(value, resultValue))) === true) {
                                exists = true;
                                break;
                            }
                        }
                        if (exists === false) {
                            yield yield tslib_1.__await(value);
                            result.push(value);
                        }
                    }
                }
                catch (e_1_1) { e_1 = { error: e_1_1 }; }
                finally {
                    try {
                        if (source_1_1 && !source_1_1.done && (_a = source_1.return)) yield tslib_1.__await(_a.call(source_1));
                    }
                    finally { if (e_1) throw e_1.error; }
                }
            }
        });
    }
    return new BasicAsyncEnumerable_1.BasicAsyncEnumerable(iterator);
}
exports.unionAsync = unionAsync;
