"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const asAsyncSortedKeyValues_1 = require("./_ordered/asAsyncSortedKeyValues");
const asAsyncSortedKeyValuesSync_1 = require("./_ordered/asAsyncSortedKeyValuesSync");
const asSortedKeyValues_1 = require("./_ordered/asSortedKeyValues");
const asSortedKeyValuesSync_1 = require("./_ordered/asSortedKeyValuesSync");
const BasicParallelEnumerable_1 = require("./BasicParallelEnumerable");
/**
 * Ordered Parallel Enumerable
 * @private
 */
class OrderedParallelEnumerable extends BasicParallelEnumerable_1.BasicParallelEnumerable {
    constructor(orderedPairs) {
        super({
            generator: () => tslib_1.__awaiter(this, void 0, void 0, function* () {
                var e_1, _a;
                const asyncVals = orderedPairs();
                const array = [];
                try {
                    for (var asyncVals_1 = tslib_1.__asyncValues(asyncVals), asyncVals_1_1; asyncVals_1_1 = yield asyncVals_1.next(), !asyncVals_1_1.done;) {
                        const val = asyncVals_1_1.value;
                        array.push(...val);
                    }
                }
                catch (e_1_1) { e_1 = { error: e_1_1 }; }
                finally {
                    try {
                        if (asyncVals_1_1 && !asyncVals_1_1.done && (_a = asyncVals_1.return)) yield _a.call(asyncVals_1);
                    }
                    finally { if (e_1) throw e_1.error; }
                }
                return array;
            }),
            type: 0 /* PromiseToArray */,
        });
        this.orderedPairs = orderedPairs;
    }
    static generateAsync(source, keySelector, ascending, comparer) {
        let orderedPairs;
        if (source instanceof OrderedParallelEnumerable) {
            orderedPairs = function () {
                return tslib_1.__asyncGenerator(this, arguments, function* () {
                    var e_2, _a;
                    try {
                        for (var _b = tslib_1.__asyncValues(source.orderedPairs()), _c; _c = yield tslib_1.__await(_b.next()), !_c.done;) {
                            const pair = _c.value;
                            yield tslib_1.__await(yield* tslib_1.__asyncDelegator(tslib_1.__asyncValues(asAsyncSortedKeyValuesSync_1.asAsyncSortedKeyValuesSync(pair, keySelector, ascending, comparer))));
                        }
                    }
                    catch (e_2_1) { e_2 = { error: e_2_1 }; }
                    finally {
                        try {
                            if (_c && !_c.done && (_a = _b.return)) yield tslib_1.__await(_a.call(_b));
                        }
                        finally { if (e_2) throw e_2.error; }
                    }
                });
            };
        }
        else {
            orderedPairs = () => asAsyncSortedKeyValues_1.asAsyncSortedKeyValues(source, keySelector, ascending, comparer);
        }
        return new OrderedParallelEnumerable(orderedPairs);
    }
    static generate(source, keySelector, ascending, comparer) {
        let orderedPairs;
        if (source instanceof OrderedParallelEnumerable) {
            orderedPairs = function () {
                return tslib_1.__asyncGenerator(this, arguments, function* () {
                    var e_3, _a;
                    try {
                        for (var _b = tslib_1.__asyncValues(source.orderedPairs()), _c; _c = yield tslib_1.__await(_b.next()), !_c.done;) {
                            const pair = _c.value;
                            yield tslib_1.__await(yield* tslib_1.__asyncDelegator(tslib_1.__asyncValues(asSortedKeyValuesSync_1.asSortedKeyValuesSync(pair, keySelector, ascending, comparer))));
                        }
                    }
                    catch (e_3_1) { e_3 = { error: e_3_1 }; }
                    finally {
                        try {
                            if (_c && !_c.done && (_a = _b.return)) yield tslib_1.__await(_a.call(_b));
                        }
                        finally { if (e_3) throw e_3.error; }
                    }
                });
            };
        }
        else {
            orderedPairs = () => asSortedKeyValues_1.asSortedKeyValues(source, keySelector, ascending, comparer);
        }
        return new OrderedParallelEnumerable(orderedPairs);
    }
    thenBy(keySelector, comparer) {
        return OrderedParallelEnumerable.generate(this, keySelector, true, comparer);
    }
    thenByAsync(keySelector, comparer) {
        return OrderedParallelEnumerable.generateAsync(this, keySelector, true, comparer);
    }
    thenByDescending(keySelector, comparer) {
        return OrderedParallelEnumerable.generate(this, keySelector, false, comparer);
    }
    thenByDescendingAsync(keySelector, comparer) {
        return OrderedParallelEnumerable.generateAsync(this, keySelector, false, comparer);
    }
}
exports.OrderedParallelEnumerable = OrderedParallelEnumerable;
