"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const _nextIterationAsync_1 = require("./_nextIterationAsync");
/**
 * Determines whether any element of a sequence satisfies a condition.
 * @param source An IEnumerable<T> whose elements to apply the predicate to.
 * @param predicate A function to test each element for a condition.
 */
function anyAsync(source, predicate) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const nextIter = _nextIterationAsync_1.nextIterationAsync(source, predicate);
        switch (nextIter.type) {
            case 0 /* PromiseToArray */:
                return nextIter.generator().then((values) => {
                    return values.some((x) => x);
                });
            case 1 /* ArrayOfPromises */:
                return Promise.all(nextIter.generator()).then((values) => {
                    return values.some((x) => x);
                });
            case 2 /* PromiseOfPromises */:
                return nextIter.generator().then((values) => Promise.all(values)).then((values) => {
                    return values.some((x) => x);
                });
        }
    });
}
exports.anyAsync = anyAsync;
