"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const StrictEqualityComparer_1 = require("../../shared/StrictEqualityComparer");
const BasicParallelEnumerable_1 = require("../BasicParallelEnumerable");
/**
 * Returns distinct elements from a sequence by using the default or specified equality comparer to compare values.
 * @param source The sequence to remove duplicate elements from.
 * @param comparer An IEqualityComparer<T> to compare values. Optional. Defaults to Strict Equality Comparison.
 * @returns An IParallelEnumerable<T> that contains distinct elements from the source sequence.
 */
function distinct(source, comparer = StrictEqualityComparer_1.StrictEqualityComparer) {
    const generator = () => tslib_1.__awaiter(this, void 0, void 0, function* () {
        const distinctElements = [];
        for (const item of yield source.toArray()) {
            const foundItem = distinctElements.find((x) => comparer(x, item));
            if (!foundItem) {
                distinctElements.push(item);
            }
        }
        return distinctElements;
    });
    return new BasicParallelEnumerable_1.BasicParallelEnumerable({
        generator,
        type: 0 /* PromiseToArray */,
    });
}
exports.distinct = distinct;
