"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
/**
 * Returns the element at a specified index in a sequence or a default value if the index is out of range.
 * @param source An IEnumerable<T> to return an element from.
 * @param index The zero-based index of the element to retrieve.
 * @returns
 * default(TSource) if the index is outside the bounds of the source sequence;
 * otherwise, the element at the specified position in the source sequence.
 */
function elementAtOrDefault(source, index) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const dataFunc = source.dataFunc;
        switch (dataFunc.type) {
            case 0 /* PromiseToArray */:
                return dataFunc.generator().then((values) => {
                    if (index >= values.length) {
                        return null;
                    }
                    else {
                        return values[index];
                    }
                });
            case 1 /* ArrayOfPromises */:
                return Promise.all(dataFunc.generator()).then((values) => {
                    if (index >= values.length) {
                        return null;
                    }
                    else {
                        return values[index];
                    }
                });
            case 2 /* PromiseOfPromises */:
                return dataFunc.generator().then((values) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                    if (index >= values.length) {
                        return null;
                    }
                    else {
                        return yield values[index];
                    }
                }));
        }
    });
}
exports.elementAtOrDefault = elementAtOrDefault;
