"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const toArray_1 = require("./toArray");
/**
 * Returns first element in sequence that satisfies predicate otherwise
 * returns the first element in the sequence. Returns null if no value found.
 * @param source An IParallelEnumerable<T> to return an element from.
 * @param predicate A function to test each element for a condition. Optional.
 * @returns The first element in the sequence
 * or the first element that passes the test in the specified predicate function.
 * Returns null if no value found.
 */
function firstOrDefault(source, predicate) {
    if (predicate) {
        return firstOrDefault_2(source, predicate);
    }
    else {
        return firstOrDefault_1(source);
    }
}
exports.firstOrDefault = firstOrDefault;
function firstOrDefault_1(source) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const dataFunc = source.dataFunc;
        switch (dataFunc.type) {
            case 0 /* PromiseToArray */: {
                const values = yield dataFunc.generator();
                if (values.length === 0) {
                    return null;
                }
                else {
                    return values[0];
                }
            }
            case 1 /* ArrayOfPromises */: {
                const promises = dataFunc.generator();
                if (promises.length === 0) {
                    return null;
                }
                else {
                    return yield promises[0];
                }
            }
            case 2 /* PromiseOfPromises */: {
                const promises = yield dataFunc.generator();
                if (promises.length === 0) {
                    return null;
                }
                else {
                    return yield promises[0];
                }
            }
        }
    });
}
function firstOrDefault_2(source, predicate) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const data = yield toArray_1.toArray(source);
        for (const value of data) {
            if (predicate(value) === true) {
                return value;
            }
        }
        return null;
    });
}
