"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const ErrorString_1 = require("../../shared/ErrorString");
const InvalidOperationException_1 = require("../../shared/InvalidOperationException");
const BasicParallelEnumerable_1 = require("../BasicParallelEnumerable");
const _nextIterationAsync_1 = require("./_nextIterationAsync");
/**
 * Invokes a transform function on each element of a sequence and returns the minimum value.
 * @param source A sequence of values to determine the minimum value of.
 * @param selector A transform function to apply to each element.
 * @throws {InvalidOperationException} source contains no elements.
 * @returns The minimum value in the sequence.
 */
function minAsync(source, selector) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const dataFunc = _nextIterationAsync_1.nextIterationAsync(source, selector);
        const maxInfo = yield new BasicParallelEnumerable_1.BasicParallelEnumerable(dataFunc).toArray();
        if (maxInfo.length === 0) {
            throw new InvalidOperationException_1.InvalidOperationException(ErrorString_1.ErrorString.NoElements);
        }
        return Math.min.apply(null, maxInfo);
    });
}
exports.minAsync = minAsync;
