"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const StrictEqualityComparer_1 = require("../../shared/StrictEqualityComparer");
/**
 * Compares two parallel sequences to see if they are equal using a comparer function.
 * @param first First Sequence
 * @param second Second Sequence
 * @param comparer Comparer
 * @returns Whether or not the two iterations are equal
 */
function sequenceEquals(
// tslint:disable-next-line:no-shadowed-variable
first, second, comparer = StrictEqualityComparer_1.StrictEqualityComparer) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const firstArray = yield first.toArray();
        const secondArray = yield second.toArray();
        if (firstArray.length !== secondArray.length) {
            return false;
        }
        for (let i = 0; i < firstArray.length; i++) {
            const firstResult = firstArray[i];
            const secondResult = secondArray[i];
            if (comparer(firstResult, secondResult) === false) {
                return false;
            }
        }
        return true;
    });
}
exports.sequenceEquals = sequenceEquals;
