"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
/**
 * Compares two parallel iterables to see if they are equal using a async comparer function.
 * @param first First Sequence
 * @param second Second Sequence
 * @param comparer Async Comparer
 * @returns Whether or not the two iterations are equal
 */
function sequenceEqualsAsync(
// tslint:disable-next-line:no-shadowed-variable
first, second, comparer) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const firstArray = yield first.toArray();
        const secondArray = yield second.toArray();
        if (firstArray.length !== secondArray.length) {
            return false;
        }
        for (let i = 0; i < firstArray.length; i++) {
            const firstResult = firstArray[i];
            const secondResult = secondArray[i];
            if ((yield comparer(firstResult, secondResult)) === false) {
                return false;
            }
        }
        return true;
    });
}
exports.sequenceEqualsAsync = sequenceEqualsAsync;
