"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const BasicParallelEnumerable_1 = require("../BasicParallelEnumerable");
/**
 * Produces the set union of two sequences by using scrict equality comparison or a specified IEqualityComparer<T>.
 * @param first An IAsyncParallel<T> whose distinct elements form the first set for the union.
 * @param second An IAsyncParallel<T> whose distinct elements form the second set for the union.
 * @param comparer The IEqualityComparer<T> to compare values. Optional.
 * @returns An IParallelEnumerable<T> that contains the elements from both input sequences, excluding duplicates.
 */
function union(first, second, comparer) {
    if (comparer) {
        return union_2(first, second, comparer);
    }
    else {
        return union_1(first, second);
    }
}
exports.union = union;
function union_1(first, second) {
    function generator() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            var e_1, _a;
            const set = new Set();
            const secondPromise = second.toArray();
            try {
                for (var first_1 = tslib_1.__asyncValues(first), first_1_1; first_1_1 = yield first_1.next(), !first_1_1.done;) {
                    const item = first_1_1.value;
                    if (set.has(item) === false) {
                        set.add(item);
                    }
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (first_1_1 && !first_1_1.done && (_a = first_1.return)) yield _a.call(first_1);
                }
                finally { if (e_1) throw e_1.error; }
            }
            const secondValues = yield secondPromise;
            for (const item of secondValues) {
                if (set.has(item) === false) {
                    set.add(item);
                }
            }
            return [...set.keys()];
        });
    }
    return new BasicParallelEnumerable_1.BasicParallelEnumerable({
        generator,
        type: 0 /* PromiseToArray */,
    });
}
function union_2(
// tslint:disable-next-line:no-shadowed-variable
first, second, comparer) {
    const generator = () => tslib_1.__awaiter(this, void 0, void 0, function* () {
        const result = [];
        const values = yield Promise.all([first.toArray(), second.toArray()]);
        for (const source of values) {
            for (const value of source) {
                let exists = false;
                for (const resultValue of result) {
                    if (comparer(value, resultValue) === true) {
                        exists = true;
                        break;
                    }
                }
                if (exists === false) {
                    result.push(value);
                }
            }
        }
        return result;
    });
    return new BasicParallelEnumerable_1.BasicParallelEnumerable({
        generator,
        type: 0 /* PromiseToArray */,
    });
}
