"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
/**
 * Paritions the Iterable<T> into a tuple of failing and passing arrays
 * based on the predicate.
 * @param source Elements to Partition
 * @param predicate Async Pass / Fail condition
 * @returns Promise of [pass, fail]
 */
function partitionAsync(source, predicate) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const fail = [];
        const pass = [];
        for (const value of source) {
            if ((yield predicate(value)) === true) {
                pass.push(value);
            }
            else {
                fail.push(value);
            }
        }
        return [pass, fail];
    });
}
exports.partitionAsync = partitionAsync;
