"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const from_1 = require("../../async/_private/from");
/**
 * Projects each element of a sequence to an IAsyncEnumerable<T> and flattens the resulting sequences into one sequence.
 * @param source A sequence of values to project.
 * @param selector A transform function to apply to each element.
 * @returns An IAsyncEnumerable<T> whose elements are the result of invoking the
 * one-to-many transform function on each element of the input sequence.
 */
function selectManyAsync(source, selector) {
    if (selector.length === 1) {
        return selectManyAsync1(source, selector);
    }
    else {
        return selectManyAsync2(source, selector);
    }
}
exports.selectManyAsync = selectManyAsync;
const selectManyAsync1 = (source, selector) => {
    function generator() {
        return tslib_1.__asyncGenerator(this, arguments, function* generator_1() {
            for (const value of source) {
                const innerValues = yield tslib_1.__await(selector(value));
                for (const innerValue of innerValues) {
                    yield yield tslib_1.__await(innerValue);
                }
            }
        });
    }
    return from_1.from(generator);
};
const selectManyAsync2 = (source, selector) => {
    function generator() {
        return tslib_1.__asyncGenerator(this, arguments, function* generator_2() {
            let index = 0;
            for (const value of source) {
                const innerValues = yield tslib_1.__await(selector(value, index));
                for (const innerValue of innerValues) {
                    yield yield tslib_1.__await(innerValue);
                }
                index++;
            }
        });
    }
    return from_1.from(generator);
};
