"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const from_1 = require("../../async/_private/from");
/**
 * Filters a sequence of values based on a predicate.
 * Each element's index is used in the logic of the predicate function.
 * @param source An Iterable<T> to filter.
 * @param predicate A async function to test each source element for a condition;
 * the second parameter of the function represents the index of the source element.
 * @returns An IAsyncEnumerable<T> that contains elements from the input sequence that satisfy the condition.
 */
function whereAsync(source, predicate) {
    if (predicate.length === 1) {
        return whereAsync_1(source, predicate);
    }
    else {
        return whereAsync_2(source, predicate);
    }
}
exports.whereAsync = whereAsync;
function whereAsync_1(source, predicate) {
    function generator() {
        return tslib_1.__asyncGenerator(this, arguments, function* generator_1() {
            for (const item of source) {
                if ((yield tslib_1.__await(predicate(item))) === true) {
                    yield yield tslib_1.__await(item);
                }
            }
        });
    }
    return from_1.from(generator);
}
function whereAsync_2(source, predicate) {
    function generator() {
        return tslib_1.__asyncGenerator(this, arguments, function* generator_2() {
            let i = 0;
            for (const item of source) {
                if ((yield tslib_1.__await(predicate(item, i++))) === true) {
                    yield yield tslib_1.__await(item);
                }
            }
        });
    }
    return from_1.from(generator);
}
