import { ErrorString } from "../../shared/ErrorString";
import { InvalidOperationException } from "../../shared/InvalidOperationException";
export function aggregate(source, seedOrFunc, func, resultSelector) {
    if (resultSelector) {
        if (!func) {
            throw new ReferenceError(`TAccumulate function is undefined`);
        }
        return aggregate_3(source, seedOrFunc, func, resultSelector);
    }
    else if (func) {
        return aggregate_2(source, seedOrFunc, func);
    }
    else {
        return aggregate_1(source, seedOrFunc);
    }
}
async function aggregate_1(source, func) {
    let aggregateValue;
    for await (const value of source) {
        if (aggregateValue) {
            aggregateValue = func(aggregateValue, value);
        }
        else {
            aggregateValue = value;
        }
    }
    if (aggregateValue === undefined) {
        throw new InvalidOperationException(ErrorString.NoElements);
    }
    return aggregateValue;
}
async function aggregate_2(source, seed, func) {
    let aggregateValue = seed;
    for await (const value of source) {
        aggregateValue = func(aggregateValue, value);
    }
    return aggregateValue;
}
async function aggregate_3(source, seed, func, resultSelector) {
    let aggregateValue = seed;
    for await (const value of source) {
        aggregateValue = func(aggregateValue, value);
    }
    return resultSelector(aggregateValue);
}
